from pathlib import Path
from autodocify_cli.lib.utils import (
    file_excluded,
    is_git_initialized,
    warn_untracked_files,
    get_git_tracked_files,
)
from typing import List
import typer

# Define the exclusion list
EXCLUDED_FILES = {
    "*.env",
    "*.log",
    "*.tmp",
    "*.bak",
    "*.swp",
    "*.swo",
    "*.orig",
    "*.iml",
    "*.key",
    "*.pem",
    "*.crt",
    "*.p12",
    "*.py[cod]",
    "*.class",
    "*.exe",
    "*.dll",
    "*.so",
    "*.tgz",
    "*.gz",
    "*.DS_Store",
    "*.AppleDouble",
    "*.LSOverride",
    "npm-debug.log*",
    "*.pytest_cache",
    ".nyc_output",
    "go.sum",
    "*.h5",
    "*.pb",
    "*.ckpt",
}

EXCLUDED_FOLDERS = {
    "node_modules",
    "dist",
    "build",
    "out",
    "__pycache__",
    "logs",
    ".vscode",
    ".idea",
    ".git",
    ".next",
    ".parcel-cache",
    ".expo",
    ".firebase",
    ".docker",
    "test-results",
    "coverage",
    ".sass-cache",
    "debug",
    "tmp",
    ".trash",
    ".bundle",
    "target",
    "vendor",
    ".settings",
}


# Function to build a trie for folder exclusions
class TrieNode:
    def __init__(self):
        self.children = {}
        self.is_end = False


class Trie:
    def __init__(self):
        self.root = TrieNode()

    def insert(self, path: str):
        node = self.root
        for part in path.split("/"):
            if part not in node.children:
                node.children[part] = TrieNode()
            node = node.children[part]
        node.is_end = True

    def starts_with(self, path: str) -> bool:
        node = self.root
        for part in path.split("/"):
            if part not in node.children:
                return False
            node = node.children[part]
        return True


# Build the folder exclusion trie
folder_trie = Trie()
for folder in EXCLUDED_FOLDERS:
    folder_trie.insert(folder)


def get_project_files(base_path: Path) -> List[str]:
    """
    Retrieve project files either from a Git repository or general directory structure.
    """
    if is_git_initialized(base_path):
        warn_untracked_files(base_path)
        tracked_files = get_git_tracked_files(base_path)
        if not tracked_files:
            typer.echo("No tracked files found in the repository.")
        return tracked_files
    else:
        typer.echo(
            "Warning: The provided directory is not a Git repository. Falling back to general file scanning."
        )
        return get_general_project_files(base_path)


# Efficient file and folder filtering
def get_general_project_files(base_path: Path) -> list[str]:
    """
    Get a list of project files, excluding files and folders based on predefined rules.
    """
    all_files = []

    for file in base_path.rglob("*"):
        # Skip directories based on the folder trie
        if file.is_dir() and folder_trie.starts_with(str(file.relative_to(base_path))):
            continue

        # Skip files based on excluded patterns
        if file.is_file():
            if file_excluded(file):
                continue
            all_files.append(str(file.relative_to(base_path)))

    return all_files
