import os
import subprocess
from pathlib import Path
from typing import List, Optional
import typer
from autodocify_cli.lib.prompt_templates.readme_prompt_template import (
    readme_prompt_template,
)
from autodocify_cli.lib.prompt_templates.technical_doc_prompt_template import (
    technical_doc_prompt_template,
)
from autodocify_cli.lib.prompt_templates.test_prompt_template import (
    test_doc_prompt_template,
)
from autodocify_cli.lib.prompt_templates.swagger_prompt_template import (
    swagger_doc_prompt_template,
)
from autodocify_cli.core.settings.config import LLMEnum, SwaggerFormatEnum

from autodocify_cli.lib.variables import (
    DEFAULT_EXCLUDED_EXTENSIONS,
    DEFAULT_EXCLUDED_PATHS,
    EXCLUDED_FILES,
    EXCLUDED_FOLDERS,
)


def greet_user(name: str = "Comrade") -> str:
    """
    Returns a greeting message.
    """
    return f"Hello, {name}! Welcome to AutoDocify."


def get_base_path(base_dir: Optional[str] = None) -> Path:
    """
    Returns the base path for the project directory or raises an error if invalid.
    """
    base_path = Path(base_dir or Path.cwd())
    if not base_path.exists():
        typer.echo(f"Error: The directory {base_path} does not exist.")
        raise typer.Exit(code=1)
    return base_path


def is_git_initialized(base_path: Path) -> bool:
    """
    Check if the directory is a Git repository.
    """
    try:
        subprocess.run(
            ["git", "rev-parse", "--is-inside-work-tree"],
            cwd=base_path,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            check=True,
        )
        return True
    except subprocess.CalledProcessError:
        # raise GitError(f"The directory '{base_path}' is not a Git repository.")
        return False


def warn_untracked_files(base_path: Path):
    """
    Warn the user if there are untracked files in the Git repository.
    """
    try:
        result = subprocess.run(
            ["git", "status", "--porcelain"],
            cwd=base_path,
            text=True,
            capture_output=True,
            check=True,
        )
        untracked_files = [
            line[3:]
            for line in result.stdout.strip().split("\n")
            if line.startswith("??")
        ]
        if untracked_files:
            typer.echo(
                f"Warning: There {'are' if len(untracked_files) > 1 else 'is'} {len(untracked_files)} untracked {'files' if len(untracked_files) > 1 else 'file'}. Consider running 'git add'."
            )
    except subprocess.CalledProcessError:
        typer.echo("Error: Unable to check for untracked files.")


def get_git_tracked_files(
    base_path: Path,
) -> List[str]:
    """
    Retrieve a list of all files tracked by Git in the given directory,
    excluding deleted files, excluded paths, and excluded extensions.
    """
    try:
        result = subprocess.run(
            ["git", "ls-files"],
            cwd=base_path,
            text=True,
            capture_output=True,
            check=True,
        )
        tracked_files = result.stdout.strip().split("\n")

        deleted_result = subprocess.run(
            ["git", "ls-files", "--deleted"],
            cwd=base_path,
            text=True,
            capture_output=True,
            check=True,
        )
        deleted_files = deleted_result.stdout.strip().split("\n")

        valid_files = [
            file
            for file in tracked_files
            if file not in deleted_files
            and not any(file.startswith(f"{path}/") for path in DEFAULT_EXCLUDED_PATHS)
            and not any(file.endswith(ext) for ext in DEFAULT_EXCLUDED_EXTENSIONS)
            and not file_excluded(file)
        ]
        return valid_files
    except subprocess.CalledProcessError as e:
        typer.echo(f"Error retrieving tracked files: {e}")
        return []


def readme_prompt(content: str) -> str:
    """
    Generates a README prompt based on the provided content.
    """
    return f"{readme_prompt_template}\n\nHere is the project content:\n{content}"


def technical_docs_prompt(content: str) -> str:
    """
    Generates a technical documentation prompt based on the provided content.
    """
    return f"{technical_doc_prompt_template}\n\nHere is the project content:\n{content}"


def create_test_prompt(content: str) -> str:
    """
    Generates a test prompt based on the provided content.
    """
    return f"{test_doc_prompt_template}\n\nHere is the project content:\n{content}"


def swagger_prompt(content: str, format: str = "json") -> str:
    """
    Generates a swagger prompt based on the provided content.
    """
    return f"{swagger_doc_prompt_template}\n\nHere is the project content:\n{content}\n\nAnd the format it should be in:\n{format}"


def check_llm(llm: str):
    llms = [option.value for option in LLMEnum]
    if llm not in llms:
        raise typer.BadParameter(
            f"Invalid LLM Specified. Valid option(s) are {LLMEnum.get_values()}"
        )
    else:
        return llm


def check_swagger_format(format: str):
    formats = [option.value for option in SwaggerFormatEnum]
    if format not in formats:
        raise typer.BadParameter(
            f"Invalid LLM Specified. Valid option(s) are {SwaggerFormatEnum.get_values()}"
        )


def resolve_format_extension(file_name: str, format: str):
    # Split the filename into root and extension
    root, ext = os.path.splitext(file_name)

    # Remove leading dots from both the existing extension and new format
    current_ext = ext.lstrip(".")
    cleaned_format = format.lstrip(".")

    # Compare extensions case-insensitively
    if current_ext.lower() != cleaned_format.lower():
        # Construct new filename with the desired extension
        return f"{root}.{cleaned_format}"
    return file_name


def validate_output_file(path: str):
    """
    Validates file paths. Returns True if the path is a directory

    Args:
        path (str): The path to validate

    Raises:
        ValueError: For missing directories or parent directories
    """
    normalized_path = os.path.normpath(path)
    parent_dir = os.path.dirname(normalized_path)
    # Check if the parent_directory exists
    # if parent_dir and not os.path.isdir(parent_dir):
    if not os.path.exists(parent_dir) and parent_dir != "":
        return False

    # Check if the path is a file
    if not os.path.isfile(normalized_path) and not os.path.exists(normalized_path):
        with open(f"{normalized_path}", "x") as new_file:
            pass
        return normalized_path
    else:
        return normalized_path


# check if user specified files to exclude and add to EXCLUDED_FILES list
def resolve_excluded_files():
    excluded_files = EXCLUDED_FILES
    # Get the ignored files from autodocify ignore if it exists
    if os.path.exists(".autodocifyignore"):
        with open(".autodocifyignore", "r") as ignore_file:
            ignored_files = ignore_file.read().split()
            excluded_files.update(ignored_files)
    return excluded_files


# Check if a file is among the excluded file list
def file_excluded(file):
    excluded_files = resolve_excluded_files()
    if type(file) == str:
        if os.path.exists(file):
            file = Path(file)
    file_name = file.name
    file_suffix = file.suffix
    for pattern in excluded_files:
        if file_name and file_name.endswith(pattern.strip("*")):
            return True
        if file_suffix and file_suffix == pattern.strip("*"):
            return True
